/*
hsize.c Version 0.1.0  Hash counts records in database
Copyright (C) 2003  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* returns total records, total overflow */
/* total empty slots and total free blocks */
/* in database */

#include <bthash.h>

void hsize(hashfmt *hshhdr, int *rcds,
   int *ovfl, int *empty, int *freeblk)
   {
   BOOL  bResult;
   DWORD msgerr;
   LPVOID lpMsgBuf;
   int eofsw;
   DWORD rdlen;
   int tot;
   h_nodefmt *blk;
   long loc;
   long ofst;
   long hashend;
   unsigned char *p,*q;
   unsigned char buf[65536 + 16];
   hshhdr->reason = 0;
   loc = SetFilePointer(hshhdr->hndl,
      (DWORD) HDRLEN, NULL,
      FILE_BEGIN);
   if (loc == UINT_MAX)
      {
      msgerr = GetLastError();
      hshhdr->reason = SYS_ERR + msgerr;
      FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
         FORMAT_MESSAGE_FROM_SYSTEM |
         FORMAT_MESSAGE_IGNORE_INSERTS,
         NULL,           /* source */
         msgerr,
         MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
         (LPTSTR) &lpMsgBuf,
         0,              /* msglen */
         NULL);          /* insertion values */
      fprintf(stderr,"hsize: %s\n", lpMsgBuf);
      LocalFree(lpMsgBuf);
      fprintf(stderr,"hsize: "
         "SetFilePointer error\n");
      hshhdr->reason = SYS_ERR + errno;
      return;
      } /* if SetFilePointer err */
   *rcds = *ovfl = *empty = *freeblk = 0;
   tot = eofsw = 0;
   blk = (h_nodefmt *) buf;
   hashend = (long) hshhdr->hashend;
   ofst = HDRLEN;
   while (!eofsw)
      {
      bResult = ReadFile(hshhdr->hndl,
         (unsigned char *) buf,
         (DWORD) hshhdr->rcdnode,
         &rdlen, NULL);
      if (!bResult)
         {
         msgerr = GetLastError();
         hshhdr->reason = SYS_ERR + msgerr;
         FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
            FORMAT_MESSAGE_FROM_SYSTEM |
            FORMAT_MESSAGE_IGNORE_INSERTS,
            NULL,           /* source */
            msgerr,
            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
            (LPTSTR) &lpMsgBuf,
            0,              /* msglen */
            NULL);          /* insertion values */
         fprintf(stderr,"hsize: %s\n", lpMsgBuf);
         LocalFree(lpMsgBuf);
         hshhdr->reason = SYS_ERR + errno;
         return;
	 } /* read err */
      if (rdlen == (DWORD) hshhdr->rcdnode)
	 {
	 tot++;
	 p = (unsigned char *) buf
	    + hshhdr->rcdhdr
	    + hshhdr->keyofst;
	 q = (unsigned char *) p + hshhdr->keylen;
	 while (p < q)
	    {
	    if (*p != 0xff) break;
	    p++;
	    } /* for each byte in key */
	 if (ofst >= hashend
	    && p < q)
	    {
            (*ovfl)++;
	    } /* if overflow */
	 /* if record key is all ff */
	 /* count it as a free block */
	 if (p >= q)
	    (*freeblk)++;
	 else if (blk->next == UINT_MAX)
	    (*empty)++;
	 else
            (*rcds)++;
	 ofst += hshhdr->rcdnode;
	 } /* if good read */
      else if (!rdlen)
	 {
	 eofsw = 1;
         break;
	 } /* eof */
      else
	 {
         fprintf(stderr,"hsize: read error\n");
         fprintf(stderr,"hshhdr->rcdnode %d "
	    "rdlen %d\n",
	    hshhdr->rcdnode, rdlen);
         hshhdr->reason = HASH_ERR + 1201;
         return;
	 } /* read err */
      } /* for each block in db */
   } /* hsize */
